@@ -1,3 +1,4 @@
+# SPDX-License-Identifier: MIT OR Apache-2.0
 # This file is dual licensed under the terms of the Apache License, Version
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
@@ -14,9 +15,10 @@ Python 3.7 as :mod:`contextvars`.
 See :doc:`contextvars`.
 """
 
+import contextlib
 import contextvars
 
-from typing import Any, Dict, Mapping
+from typing import Any, Dict, Generator, Mapping
 
 import structlog
 
@@ -105,12 +107,12 @@ def bind_contextvars(**kw: Any) -> "Mapping[str, contextvars.Token[Any]]":
     Use this instead of :func:`~structlog.BoundLogger.bind` when you want some
     context to be global (context-local).
 
-    Return the mapping of ``contextvars.Token``\s resulting
-    from setting the backing ``ContextVar``\s.
+    Return the mapping of `contextvars.Token`\s resulting
+    from setting the backing :class:`~contextvars.ContextVar`\s.
     Suitable for passing to :func:`reset_contextvars`.
 
     .. versionadded:: 20.1.0
-    .. versionchanged:: 21.1.0 Return the ``contextvars.Token`` mapping
+    .. versionchanged:: 21.1.0 Return the `contextvars.Token` mapping
         rather than None. See also the toplevel note.
     """
     rv = {}
@@ -153,3 +155,24 @@ def unbind_contextvars(*keys: str) -> None:
         structlog_k = f"{STRUCTLOG_KEY_PREFIX}{k}"
         if structlog_k in _CONTEXT_VARS:
             _CONTEXT_VARS[structlog_k].set(Ellipsis)
+
+
+@contextlib.contextmanager
+def bound_contextvars(**kw: Any) -> Generator[None, None, None]:
+    """
+    Bind *kw* to the current context-local context. Unbind or restore *kw*
+    afterwards. Do **not** affect other keys.
+
+    Can be used as a context manager or decorator.
+
+    .. versionadded:: 21.4.0
+    """
+    context = get_contextvars()
+    saved = {k: context[k] for k in context.keys() & kw.keys()}
+
+    bind_contextvars(**kw)
+    try:
+        yield
+    finally:
+        unbind_contextvars(*kw.keys())
+        bind_contextvars(**saved)
